/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004-2005  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    $Id: tlist.c,v 1.5 2005/05/19 11:17:31 kasperd Exp $
*/

#include "tstore.h"

/* Format string is not fully implemented and semantics
 * may change in the future as more features are added.
 */

int main(int argc, char ** argv)
{
  uint8_t *buf;
  struct store *s;
  uint64_t idx,nidx;
  char *format="%s\t%p\n";
  char *storename=NULL;

  switch (argc) {
  case 2:
    storename=argv[1];
    break;
  case 4:
    if (!strcmp(argv[1],"-f")) {
      format=argv[2];
      storename=argv[3];
    } else if (!strcmp(argv[2],"-f")) {
      format=argv[3];
      storename=argv[1];
    }
    break;
  }

  if (!storename) {
    fprintf(stderr,"Uage: %s [-f format] <store>\n",argv[0]);
    return 1;
  }

  s=open_store(storename,O_RDONLY,DONT_LOCK);
  if (!s) errstop(storename);
  buf=s->header_buf;

  for (idx=GET_BNUM(buf+NEXTPTR_OFFSET); idx; idx=nidx) {
    char *fp;
    get_block(s,idx,buf);
    for (fp=format;*fp;) {
      switch(*fp) {
      case '\\':
	switch(fp[1]) {
	case '\\':
	  putchar('\\');
	  fp+=2;
	  break;
	case 'n':
	  putchar('\n');
	  fp+=2;
	  break;
	case 'r':
	  putchar('\r');
	  fp+=2;
	  break;
	case 't':
	  putchar('\t');
	  fp+=2;
	  break;
	default:
	  putchar(*(fp++));
	  break;
	}
	break;
      case '%':
 	switch(fp[1]) {
	case '%':
	  putchar('%');
	  fp+=2;
	  break;
	case 'a':
	case 'c':
	case 't':
	  printf("%lld",GET_TIME(buf+SECONDS_OFFSET));
	  fp+=2;
	  break;
	case 'N':
	  printf("%09lld",GET_NTIM(buf+NANOSEC_OFFSET));
	  fp+=2;
	  break;
	case 'p':
	  printf("%s",buf+FILENAME_OFFSET);
	  fp+=2;
	  break;
	case 's':
	  printf("%lld",GET_SIZE(buf+FILESIZE_OFFSET));
	  fp+=2;
	  break;
	default:
	  putchar(*(fp++));
	  break;
	}
	break;
     default:
	putchar(*(fp++));
	break;
      }
    }
    fflush(stdout);
    nidx=GET_BNUM(buf+NEXTPTR_OFFSET);
    if ((nidx>=s->store_size-1)||(nidx==idx)) {
      fprintf(stderr,"Invalid link %lld=>%lld\n",idx,nidx);
      return 1;
    }
  }
  return 0;
}
